@@ -1,5 +1,6 @@
 from portfoliotools.screener.utility.util import get_ticker_list, backTestStrategy
 from portfoliotools.screener.stock_detail import StockDetail
+from portfoliotools.screener.option_screener import OptionHelper
 from datetime import datetime, timedelta
 import random
 import json
@@ -292,6 +293,86 @@ class Strategy52WHigh(Strategy):
         else:
             return result
         
+# Option Strategies
+class OptionIVDiffStrategy:
+    
+    def sortStrikes(self, up, x):
+        return abs(up-x)
+
+    def getClosestStrikes(self, strikes, value, count = 3):
+        strikes.sort(key = lambda x: self.sortStrikes(value, x))
+        filtered_strikes = strikes[:count]
+        return filtered_strikes
+
+    def getAction(self, x, threshold = 10):
+        action = ''
+        if abs(x['iv_diff']) >= threshold:
+            if x['iv_diff'] > 0:
+                action = 'Sell ' + x['ticker'] + ' PUT@' + str(x['strike_low']) + ' for ' + str(x['strike_low_pe_price'])
+                action +=  ' and Buy ' + x['ticker'] + ' PUT@' + str(x['strike_low_nxt']) + ' for ' + str(x['strike_low_pe_nxt_price'])
+            else:
+                action = 'Sell ' + x['ticker'] + ' CALL@' + str(x['strike_high']) + ' for ' + str(x['strike_high_ce_price'])
+                action +=  ' and Buy ' + x['ticker'] + ' CALL@' + str(x['strike_high_nxt']) + ' for ' + str(x['strike_high_ce_nxt_price'])
+        return action
+    
+    def screener(self,nextExpiryCount = 1):
+        data = []
+        failed = []
+        helper = OptionHelper()
+        for ticker in tqdm(helper.fnoList()[:5]):
+            try:
+                df = helper.formatedOptionChain(ticker, False)
+                nearestExpiry = list(set(df.expiryDate.values))
+                nearestExpiry.sort()
+                nearestExpiry = nearestExpiry[nextExpiryCount-1]
+                df = df[df['expiryDate'].isin([nearestExpiry])]
+
+                callStrikes = list(set((df[df['iv_ce'] > 0].strikePrice.values)))
+                putStrikes = list(set((df[df['iv_pe'] > 0].strikePrice.values)))
+                strikes = list(set((df.strikePrice.values)))
+                strikes.sort(reverse = False)
+                up = df.underlyingValue.values[0]
+
+                iv_ce = round(np.mean(df[df['strikePrice'].isin(self.getClosestStrikes(callStrikes, up))].iv_ce),2)
+                iv_pe = round(np.mean(df[df['strikePrice'].isin(self.getClosestStrikes(putStrikes, up))].iv_pe),2)
+                iv = round((iv_ce + iv_pe)/2,2)
+                days_to_expiry = (datetime.strptime(nearestExpiry, '%Y-%m-%d') - datetime.today()).days
+                low = round(up*(1 - iv*np.sqrt(days_to_expiry/365)/100),2)
+                high = round(up*(1 + iv*np.sqrt(days_to_expiry/365)/100),2)
+                strike_low = self.getClosestStrikes(putStrikes, low,1)[0]
+                strike_low_nxt = strikes[strikes.index(strike_low) - 1]
+                strike_high = self.getClosestStrikes(callStrikes, high,1)[0]
+                strike_high_nxt = strikes[strikes.index(strike_high) + 1]
+                strike_low_put_price = df[df['strikePrice'].isin([strike_low])].put_price.values[0]
+                strike_low_put_nxt_price = df[df['strikePrice'].isin([strike_low_nxt])].put_price.values[0]
+                strike_high_call_price = df[df['strikePrice'].isin([strike_high])].call_price.values[0]
+                strike_high_call_nxt_price = df[df['strikePrice'].isin([strike_high_nxt])].call_price.values[0]
+                data.append({
+                    'ticker' : ticker,
+                    'iv_ce' : iv_ce,
+                    'iv_pe' : iv_pe,
+                    'underlying' : up,
+                    'days_to_expiry' : days_to_expiry,
+                    'iv' : iv,
+                    'low' : low,
+                    'high' : high,
+                    'iv_diff' : round(iv_pe - iv_ce,2),
+                    'strike_low' : strike_low,
+                    'strike_high' : strike_high,
+                    'strike_low_pe_price' : strike_low_put_price,
+                    'strike_high_ce_price' : strike_high_call_price,
+                    'strike_low_nxt' : strike_low_nxt,
+                    'strike_high_nxt' : strike_high_nxt,
+                    'strike_low_pe_nxt_price' : strike_low_put_nxt_price,
+                    'strike_high_ce_nxt_price' : strike_high_call_nxt_price
+                    })
+            except:
+                failed.append(ticker)
+        data = pd.DataFrame(data)
+        data['Action'] = data.apply(lambda x: self.getAction(x), axis = 1)
+        return data
+ 
+# Screener Strategies       
 class ScreenerData:
     
     def __init__(self, url = None):
